function [b,a] = makeFilter(Difficulty, Type)
%% 1st and 2nd order filters, either FIR or all-poles. real systems, minimum-phase
if Difficulty == 1
    mag = rand(1)*0.6+0.3;          % mag is between 0.3 and 0.9
    phase = pi/2*rand(1) + pi/4;    % phase is between pi/4 and 3*pi/4
    switch Type
        case 1      % all-pole filter with complex conjugated poles
            b=1; a=poly([mag*exp(1i*phase) mag*exp(-1i*phase)]);
        case 2  % minimum-phase FIR with complex conjugated zeros
            a=1; b=poly([mag*exp(1i*phase) mag*exp(-1i*phase)]);
        case 3  % minimum-phase FIR with real zeros
            a=1; b=poly([mag -phase/pi]);
        case 4  % all-pole filter with real poles
            b=1; a=poly([mag -phase/pi]);
        case 5  % 1st order all-pole
            b=1; a=[1 mag];
        case 6  % 1st order all-pole
            b=1; a=[1 -mag];
        case 7  % 1st order FIR
            a=1; b=[1 mag];
        case 8  % 1st order FIR
            a=1; b=[1 -mag];
    end
%% 2nd order filters, IIR    
elseif Difficulty==2    
    mag1=rand(1)*0.6+0.3;    % mag is between 0.3 and 0.9
    phase1=pi/2*rand(1)+pi/4;       % phase is between pi/4 and 3*pi/4
    mag2=rand(1)*0.6+0.3;    % mag is between 0.3 and 0.9
    phase2=pi/2*rand(1)+pi/4;       % phase is between pi/4 and 3*pi/4
    sgn=sign(rand(1)-0.5);
    switch Type
        case 1      % 2nd order filter, complex conjugate poles and zeros
            b=poly([mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
            a=poly([mag2*exp(1i*phase2) mag2*exp(-1i*phase2)]);
        case 2  % 2nd order filter, complex conjugate zeros, real poles
            b=poly([mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
            a=poly([mag2 -phase2/pi]);
        case 3  % 2nd order filter, complex conjugate poles, real zeros
            a=poly([mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
            b=poly([mag2 -phase2/pi]);
        case 4  % 2nd order filter, maximum-phase, complex conjugate zeros, real poles
            b=poly([min(3,1/mag1)*exp(1i*phase1) min(3,1/mag1)*exp(-1i*phase1)]);
            a=poly([mag2 -phase2/pi]);
        case 5  % 2nd order filter, maximum-phase, complex conjugate zeros and poles
            b=poly([min(3,1/mag1)*exp(1i*phase1) min(3,1/mag1)*exp(-1i*phase1)]);
            a=poly([mag2*exp(1i*phase2) mag2*exp(-1i*phase2)]);
        case 6  % 2nd order filter, maximum-phase, complex conjugate poles, real zeros
            a=poly([mag2*exp(1i*phase2) mag2*exp(-1i*phase2)]);
            b=poly([min(3,1/mag1) sgn*min(3,1/(phase1/pi))]);     
        case 7  % 2nd order filter, complex conjugate poles, real zeros
            b=poly([mag1 -min(3,1/(phase1/pi))]);    
            a=poly([mag2*exp(1i*phase2) mag2*exp(-1i*phase2)]);
        case 8  % 2nd order filter, complex conjugate poles, real zeros
            b=poly([mag1 min(3,1/(phase1/pi))]);    
            a=poly([mag2*exp(1i*phase2) mag2*exp(-1i*phase2)]);
    end
%% filters with higher order, notch filters
elseif Difficulty==3
    mag1=rand(1)*0.6+0.3;           % mag is between 0.3 and 0.9
    phase1=pi/2*rand(1)+pi/4;       % phase is between pi/4 and 3*pi/4
    mag2=rand(1)*0.6+0.3;           % mag is between 0.3 and 0.9
    phase2=pi/2*rand(1)+pi/4;       % phase is between pi/4 and 3*pi/4
    mag3=rand(1)*0.6+0.3;           % mag is between 0.3 and 0.9
    phase3=pi/2*rand(1)+pi/4;       % phase is between pi/4 and 3*pi/4
    switch Type
        case 1      % Notch Filter (FIR) (2nd)
            b=poly([exp(1i*phase2) exp(-1i*phase2)]);
            a=1;
        case 2  % Notch Filter (FIR) (4th)
            b=poly([exp(1i*phase2) exp(-1i*phase2) exp(1i*phase1) exp(-1i*phase1)]);
            a=1;
        case 3 % 4th order filter, minimum phase
            b=poly([mag2*exp(1i*phase2) mag2*exp(-1i*phase2) mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
            a=poly([mag3*exp(1i*phase3) mag3*exp(-1i*phase3)]);
        case 4 % 4th order filter, minimum phase
            a=poly([mag2*exp(1i*phase2) mag2*exp(-1i*phase2) mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
            b=poly([mag3*exp(1i*phase3) mag3*exp(-1i*phase3)]);
        case 5 % 4th order filter, maximum phase
            a=poly([mag2*exp(1i*phase2) mag2*exp(-1i*phase2) mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
            b=poly([min(3,1/mag3)*exp(1i*phase3) min(3,1/mag3)*exp(-1i*phase3)]);
        case 6 % 4th order filter, maximum phase
            b=poly([min(3,1/mag2)*exp(1i*phase2) min(3,1/mag2)*exp(-1i*phase2) min(3,1/mag1)*exp(1i*phase1) min(3,1/mag1)*exp(-1i*phase1)]);
            a=poly([mag3*exp(1i*phase3) mag3*exp(-1i*phase3)]);
        case 7 % 4th order filter
            b=poly([min(3,1/mag2)*exp(1i*phase2) min(3,1/mag2)*exp(-1i*phase2) mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
            a=poly([mag3*exp(1i*phase3) mag3*exp(-1i*phase3)]);
        case 8 % Notch filter with poles
            b=poly([exp(1i*phase2) exp(-1i*phase2) exp(1i*phase1) exp(-1i*phase1)]);
            a=poly([mag3*exp(1i*phase3) mag3*exp(-1i*phase3)]);
    end
%% filters with higher order, complicated notch filters, complex systems, all-pass components
elseif Difficulty==4
    mag1=rand(1)*0.6+0.3;           % mag is between 0.3 and 0.9
    phase1=pi/2*rand(1)+pi/4;       % phase is between pi/4 and 3*pi/4
    mag2=rand(1)*0.6+0.3;           % mag is between 0.3 and 0.9
    phase2=pi/2*rand(1)+pi/4;       % phase is between pi/4 and 3*pi/4
    mag3=rand(1)*0.6+0.3;           % mag is between 0.3 and 0.9
    %phase3=pi/2*rand(1)+pi/4;       % phase is between pi/4 and 3*pi/4
    sgn=sign(rand(1)-0.5);
    switch Type
        case 1      % 2nd order all-pass
            b=poly([1/mag1*exp(1i*phase1) 1/mag1*exp(-1i*phase1)]);
            a=poly([mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
        case 2  % notch-like filter
            a=poly([min(mag1,mag2)*exp(1i*phase1) min(mag1,mag2)*exp(-1i*phase1)]);
            b=poly([max(mag1,mag2)*exp(1i*phase1) max(mag1,mag2)*exp(-1i*phase1)]);
        case 3  % double-complex filter
            a=poly([sgn*mag1*exp(1i*phase1) -sgn*mag1*exp(-1i*phase1)]);
            b=poly([sgn*mag2*exp(1i*phase2) -sgn*mag2*exp(-1i*phase2)]);
        case 4  % complex filter with all-pass component
            b=poly([1/mag1*exp(1i*phase1) sgn*mag2]);
            a=poly([mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
        case 5  % complex filter
            b=poly([1/mag1*exp(1i*phase1) mag2*exp(1i*phase2) mag2*exp(-1i*phase2)]);
            a=poly([0 0 sgn*mag3]);
        case 6  % non-allpass filter
            b=poly([1/mag2*exp(1i*phase1) 1/mag2*exp(-1i*phase1) 0]);
            a=poly([mag1*exp(1i*phase1) mag1*exp(-1i*phase1) sgn*mag3]);
        case 7  % non-allpass filter
            b=poly([1/mag2*exp(1i*phase1) 1/mag2*exp(-1i*phase1)]);
            a=poly([mag1*exp(1i*phase1) mag1*exp(-1i*phase1)]);
        case 8  % real filter, 3rd order
            b=poly([-mag2*exp(1i*phase1) -mag2*exp(-1i*phase1) 0]);
            a=poly([mag1*exp(1i*phase1) mag1*exp(-1i*phase1) sgn*mag3]);
    end
else
    disp('Something is wrong here...invalid difficulty level')
    a=1;b=1;
end